# calculate 7_159_090.9 / CLKFRQ << 32
#
# This is the same as:
#
#    ((7_159_090.9 * 10) * 2^32) / (CLKFRQ * 10)
#
# So we calculate the quotient of dividing a 32=bit number into a 64-bit number.
#
# (71_590_909 * 2^32) / (CLKFRQ * 10)
#
# Since the low-order 32 bits of the 64-bit value are zero, we don't really have
# to store them -- we just shift zeros in during the subtraction loop.

use strict;
use warnings;

# We can't "use integer;" here because 0x8000_0000 >> 1 does not become 0x4000_0000,
# it becomes "0xC000_0000 (it preserves the sign of the result.
# But in assembly language we can control that.
#use integer;

sub DoCalc($)
{
    my $CLKFRQ = shift;

    my $dividend = 71_590_909 << 1; # the "<< 1" adjusts for the first bit,
                                    # because we assume that $dividend < $divisor
                                    # this is (7_159_090.9 * 10) << 1
    my $divisor  = $CLKFRQ * 10;

    my $quotient = 0;
    my $bit      = 0x8000_0000;     # because we assume that $dividend < $divisor,
                                    # see above.

    while ( $bit != 0 )
    {
        if ( $dividend >= $divisor )
        {
            $dividend -= $divisor;
            $quotient |= $bit;
        }
        $bit      >>= 1;
        $dividend <<= 1;
    }

    return $quotient;
}

# Test program:

my $CLK = 100_000_000;
my $num = DoCalc( $CLK );
printf "For clock freq \%9u computed: 0x\%.8X\n",
       $CLK, $num;

$CLK =  96_000_000;
$num = DoCalc( $CLK );
printf "For clock freq \%9u computed: 0x\%.8X\n",
       $CLK, $num;

exit 0;
